package at.xaxa.ledger.ui.category.edit

import android.util.Log
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Info
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.ui.AppViewModelProvider
import at.xaxa.ledger.ui.ButtonDanger
import at.xaxa.ledger.ui.ButtonSuccess
import at.xaxa.ledger.ui.category.iconNames
import at.xaxa.ledger.ui.category.icons



@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun EditCategory(
    onButtonClick: () -> Unit,
    modifier: Modifier = Modifier,
    editCategoryViewModel: EditCategoryViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    val category = editCategoryViewModel.categoryUi.category
    var showError by remember { mutableStateOf(editCategoryViewModel.deletePossible) }
    var redirect by remember { mutableStateOf(editCategoryViewModel.redirect) }
    var deleteStarted by remember { mutableStateOf(false) }
    var expanded by remember { mutableStateOf(false) }
    var selectedIconIndex by remember { mutableIntStateOf(category.icon) }

    LaunchedEffect(category) {
        selectedIconIndex = category.icon
    }

    LaunchedEffect(deleteStarted) {
        if (deleteStarted) {
            editCategoryViewModel.findEntryByCategoryId()
            showError = editCategoryViewModel.deletePossible

        }
        if (redirect) {
            onButtonClick()
        }
    }

    if (showError) {
        AlertDialog(
            icon = {
                Icon(Icons.Default.Info, contentDescription = "Info Icon")
            },
            title = {
                Text(text = "Category in use")
            },
            text = {
                Text(text = "You are trying to delete a Category which is still in use. Please delete or change the Category in your entries which are in use.")
            },
            onDismissRequest = {
                deleteStarted = false
                showError = false
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        deleteStarted = false
                        showError = false
                    }
                ) {
                    Text("Okay")
                }
            }
        )
    }


    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp, 0.dp),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Column(
            Modifier.weight(1f)
        ) {
            OutlinedTextField(
                value = category.categoryName,
                onValueChange = { editCategoryViewModel.updateCategory(category.copy(categoryName = it)) },
                label = { Text("Category Name") },
                modifier = Modifier
                    .fillMaxWidth()
            )

            ExposedDropdownMenuBox(
                expanded = expanded,
                onExpandedChange = { expanded = it }
            ) {
                OutlinedTextField(
                    value = iconNames[selectedIconIndex],
                    onValueChange = {},
                    label = { Text("Icon") },
                    readOnly = true,
                    leadingIcon = {
                        Icon(
                            imageVector = icons[selectedIconIndex],
                            contentDescription = "Leading Icon"
                        )
                    },
                    trailingIcon = {
                        ExposedDropdownMenuDefaults.TrailingIcon(expanded = expanded)
                    },
                    modifier = Modifier
                        .menuAnchor()
                        .fillMaxWidth()
                )

                ExposedDropdownMenu(
                    expanded = expanded,
                    onDismissRequest = { expanded = false }
                ) {
                    icons.forEachIndexed { index, icon ->
                        DropdownMenuItem(
                            text = { Text(text = iconNames[index]) },
                            onClick = {
                                selectedIconIndex = index
                                editCategoryViewModel.updateCategory(category.copy(icon = index))
                                expanded = false
                            },
                            leadingIcon = {
                                Icon(
                                    imageVector = icon,
                                    contentDescription = iconNames[index]
                                )
                            }
                        )
                    }
                }
            }
        }

        Column(
            modifier = Modifier.fillMaxWidth(),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            ButtonDanger(
                modifier = Modifier
                    .padding(bottom = 8.dp),
                "Delete Category",
                onClick = {
                    deleteStarted = true
                    editCategoryViewModel.findEntryByCategoryId()
                }
            )
            ButtonSuccess(
                modifier = Modifier,
                "Save Category",
                onClick = {
                    editCategoryViewModel.saveCategory()
                    onButtonClick()
                }
            )
        }
    }
}

